@@ -5,12 +5,19 @@ try:
 except ImportError:
     # Django 1.3
     from django.conf.urls.defaults import patterns, include
+from django.core.exceptions import ValidationError
 from django.contrib import admin
 from django.contrib.admin.views.main import ChangeList
 from django.contrib.admin.sites import AdminSite
 from django.contrib.auth.models import User
 from django.db import models
-from django.test import TestCase, RequestFactory
+from django.test import TestCase
+
+try:
+    # django >= 3
+    from django.test import RequestFactory
+except ImportError:
+    from safedelete.test_utils import RequestFactory
 
 from .admin import SafeDeleteAdmin, highlight_deleted
 from .models import (safedelete_mixin_factory, SoftDeleteMixin,
@@ -20,13 +27,18 @@ from .models import (safedelete_mixin_factory, SoftDeleteMixin,
 
 # MODELS (FOR TESTING)
 
+if django.VERSION[1] < 3:
+    AUTHOR_POLICY = HARD_DELETE
+else:
+    AUTHOR_POLICY = HARD_DELETE_NOCASCADE
 
-class Author(safedelete_mixin_factory(HARD_DELETE_NOCASCADE)):
+
+class Author(safedelete_mixin_factory(AUTHOR_POLICY)):
     name = models.CharField(max_length=200)
 
 
 class Category(safedelete_mixin_factory(SOFT_DELETE, visibility=DELETED_VISIBLE_BY_PK)):
-    name = models.CharField(max_length=200)
+    name = models.CharField(max_length=200, unique=True)
 
 
 class Article(safedelete_mixin_factory(HARD_DELETE)):
@@ -62,31 +74,24 @@ class SimpleTest(TestCase):
     def setUp(self):
 
         self.authors = (
-            Author(name='author 0'),
-            Author(name='author 1'),
-            Author(name='author 2'),
+            Author.objects.create(name='author 0'),
+            Author.objects.create(name='author 1'),
+            Author.objects.create(name='author 2'),
         )
 
         self.categories = (
-            Category(name='category 0'),
-            Category(name='category 1'),
-            Category(name='category 2'),
+            Category.objects.create(name='category 0'),
+            Category.objects.create(name='category 1'),
+            Category.objects.create(name='category 2'),
         )
 
-        for i in self.authors + self.categories:
-            i.save()
-
         self.articles = (
-            Article(name='article 0', author=self.authors[1]),
-            Article(name='article 1', author=self.authors[1], category=self.categories[1]),
-            Article(name='article 2', author=self.authors[2], category=self.categories[2]),
+            Article.objects.create(name='article 0', author=self.authors[1]),
+            Article.objects.create(name='article 1', author=self.authors[1], category=self.categories[1]),
+            Article.objects.create(name='article 2', author=self.authors[2], category=self.categories[2]),
         )
 
-        for i in self.articles:
-            i.save()
-
-        self.order = Order(name='order')
-        self.order.save()
+        self.order = Order.objects.create(name='order')
         self.order.articles.add(self.articles[0], self.articles[1])
 
     def test_softdelete(self):
@@ -116,6 +121,9 @@ class SimpleTest(TestCase):
         self.assertEqual(Article.objects.filter(author=self.authors[2]).count(), 1)
 
     def test_hard_delete_nocascade(self):
+        if django.VERSION[1] < 3:
+            return
+
         self.assertEqual(Author.objects.count(), 3)
 
         self.authors[0].delete()
@@ -196,6 +204,12 @@ class SimpleTest(TestCase):
 
         self.assertEqual(Category.objects.count(), 3)
 
+    def test_validate_unique(self):
+        """ Check that uniqueness is also checked against deleted objects """
+        Category.objects.create(name='test').delete()
+        category = Category(name='test')
+        self.assertRaises(ValidationError, category.validate_unique)
+
 
 class AdminTestCase(TestCase):
     urls = 'safedelete.tests'
@@ -247,7 +261,7 @@ class AdminTestCase(TestCase):
     def test_admin_model(self):
         changelist_default = self.get_changelist(self.request, Category, self.modeladmin_default)
         changelist = self.get_changelist(self.request, Category, self.modeladmin)
-        if django.VERSION[1] == 3:
+        if django.VERSION[1] <= 3:
             # Django == 1.3
             self.assertEqual(changelist.get_filters(self.request)[0][0].title(), "deleted")
             self.assertEqual(changelist.get_query_set().count(), 3)
@@ -276,7 +290,12 @@ class AdminTestCase(TestCase):
             'action': ['undelete_selected'],
             '_selected_action': [self.categories[1].pk],
         })
-        self.assertTemplateUsed(resp, 'safedelete/undelete_selected_confirmation.html')
+
+        if django.VERSION[1] < 3:
+            self.assertTemplateUsed(resp, 'safedelete/undelete_selected_confirmation_django12.html')
+        else:
+            self.assertTemplateUsed(resp, 'safedelete/undelete_selected_confirmation.html')
+
         category = Category.objects.get(pk=self.categories[1].pk)
         self.assertTrue(self.categories[1].deleted)
 
